/**
 * https://github.com/open-telemetry/opentelemetry-specification/blob/master/specification/trace/semantic_conventions/http.md
 */
export declare enum AttributeNames {
    HTTP_ERROR_NAME = "http.error_name",
    HTTP_ERROR_MESSAGE = "http.error_message",
    HTTP_STATUS_TEXT = "http.status_text"
}
//# sourceMappingURL=AttributeNames.d.ts.map